from Products.Five import BrowserView

from zope import schema

from zope.annotation import factory
from zope.component import adapts, queryMultiAdapter, queryUtility
from zope.interface import Interface, implements
from zope.publisher.interfaces.browser import IBrowserRequest

from recaptcha.client.captcha import displayhtml, submit

from plone.registry.interfaces import IRegistry

from plone.formwidget.recaptcha.interfaces import IReCaptchaSettings

class IRecaptchaInfo(Interface):
    error = schema.TextLine()
    verified = schema.Bool()

class RecaptchaInfoAnnotation(object):
    implements(IRecaptchaInfo)
    adapts(IBrowserRequest)
    def __init__(self):
        self.error = None
        self.verified = False
RecaptchaInfo = factory(RecaptchaInfoAnnotation)

class RecaptchaView(BrowserView):

    def __init__(self, context, request):
        self.context = context
        self.request = request
        registry = queryUtility(IRegistry)
        self.settings = registry.forInterface(IReCaptchaSettings)

    def image_tag(self):
        portal_state = queryMultiAdapter((self.context, self.request), name=u'plone_portal_state')
        if portal_state is not None:
            lang = portal_state.language()[:2]
        else:
            lang = 'en'
        options = """
        <script>
        var RecaptchaOptions = {
            lang: '%s'
        };
        </script>
        """ % lang

        if not self.settings.public_key:
            raise ValueError, 'No recaptcha public key configured. Go to path/to/site/@@recaptcha-settings to configure.'
        use_ssl = self.request['SERVER_URL'].startswith('https://')
        error = IRecaptchaInfo(self.request).error
        return options + displayhtml(self.settings.public_key, use_ssl=use_ssl, error=error)

    def audio_url(self):
        return None

    def verify(self, input=None):
        info = IRecaptchaInfo(self.request)
        if info.verified:
            return True

        if not self.settings.private_key:
            raise ValueError, 'No recaptcha private key configured. Go to path/to/site/@@recaptcha-settings to configure.'
        challenge_field = self.request.get('recaptcha_challenge_field')
        response_field = self.request.get('recaptcha_response_field')
        remote_addr = self.request.get('HTTP_X_FORWARDED_FOR', '').split(',')[0]
        if not remote_addr:
            remote_addr = self.request.get('REMOTE_ADDR')
        res = submit(challenge_field, response_field, self.settings.private_key, remote_addr)
        if res.error_code:
            info.error = res.error_code

        info.verified = res.is_valid
        return res.is_valid

    @property
    def external(self):
        return True



from zope.i18nmessageid import MessageFactory
ReCaptchaMessageFactory = MessageFactory('plone.formwidget.recaptcha')

from plone.formwidget.recaptcha.widget import ReCaptchaWidget
from plone.formwidget.recaptcha.widget import ReCaptchaFieldWidget
from plone.formwidget.recaptcha.validator import ReCaptchaValidator


from z3c.form import interfaces

from zope import schema
from zope.interface import Interface

from zope.i18nmessageid import MessageFactory

_ = MessageFactory('plone.formwidget.recaptcha')

class IReCaptchaWidget(interfaces.IWidget):
    """Marker interface for the ReCaptcha widget
    """

class IReCaptchaSettings(Interface):
    """Global discussion settings. This describes records stored in the
    configuration registry and obtainable via plone.registry.
    """

    # Todo: Write a short hint, that other discussion related options can
    # be found elsewhere in the Plone control panel:
    #
    # - Types control panel: Allow comments on content types
    # - Search control panel: Show comments in search results

    public_key = schema.TextLine(title=_(u"Public Key"),
                                   description=_(u""),
                                   required=True,
                                   default=u"")

    private_key = schema.TextLine(title=_(u"Private Key"),
                                   description=_(u""),
                                   required=True,
                                   default=u"")


from Products.Five.browser import BrowserView

from zope.component import queryUtility

from plone.registry.interfaces import IRegistry

from plone.app.registry.browser import controlpanel

from plone.formwidget.recaptcha.interfaces import IReCaptchaSettings, _

class ReCaptchaSettingsEditForm(controlpanel.RegistryEditForm):

    schema = IReCaptchaSettings
    label = _(u"ReCaptcha settings")
    description = _(u"""In order to use ReCaptcha on your Plone site, you need
                        an account on http://recaptcha.net. Go to
                        http://recaptcha.net/whyrecaptcha.html to create an
                        account and to receive your private and public key.

                        If you don't want to rely on an external service for
                        captcha, you might want to consider using
                        plone.formwidget.captcha instead.
                        """)

    def updateFields(self):
        super(ReCaptchaSettingsEditForm, self).updateFields()

    def updateWidgets(self):
        super(ReCaptchaSettingsEditForm, self).updateWidgets()

class ReCaptchaSettingsControlPanel(controlpanel.ControlPanelFormWrapper):
    form = ReCaptchaSettingsEditForm



from Acquisition import aq_inner

from z3c.form import validator

from z3c.form.interfaces import IValidator

from zope.component import getMultiAdapter, provideAdapter

from zope.schema import ValidationError

from plone.formwidget.recaptcha import ReCaptchaMessageFactory as _

class WrongCaptchaCode(ValidationError):
    __doc__ = _("""The code you entered was wrong, please enter the new one.""")

class ReCaptchaValidator(validator.SimpleFieldValidator):

    def validate(self, value):
        super(ReCaptchaValidator, self).validate(value)
        captcha = getMultiAdapter((aq_inner(self.context), self.request), name='captcha')

        if not captcha.verify():
            raise WrongCaptchaCode
        else:
            return True



from Acquisition import aq_inner
from zope import interface
from zope import schema
from zope.component import getMultiAdapter
from z3c.form import form, field, button
from plone.z3cform.layout import wrap_form
from plone.formwidget.recaptcha.widget import ReCaptchaFieldWidget


class IReCaptchaForm(interface.Interface):
    subject = schema.TextLine(title=u"Subject",
                              description=u"",
                              required=True)

    captcha = schema.TextLine(title=u"ReCaptcha",
                              description=u"",
                              required=False)

class ReCaptcha(object):
    subject = u""
    captcha = u""
    def __init__(self, context):
        self.context = context

class BaseForm(form.Form):
    """ example captcha form """
    fields = field.Fields(IReCaptchaForm)
    fields['captcha'].widgetFactory = ReCaptchaFieldWidget

    @button.buttonAndHandler(u'Save')
    def handleApply(self, action):
        data, errors = self.extractData()
        if data.has_key('captcha'):
            # Verify the user input against the captcha
            captcha = getMultiAdapter((aq_inner(self.context), self.request), name='captcha')
            if captcha.verify(data['captcha']):
                print 'ReCaptcha validation passed.'
            else:
                print 'The code you entered was wrong, please enter the new one.'
        return

ReCaptchaForm = wrap_form(BaseForm)


import zope.component
import zope.interface
import zope.schema.interfaces

from Acquisition import aq_inner
from z3c.form import interfaces
from z3c.form import widget
from z3c.form import converter
from z3c.form.browser import text
from interfaces import IReCaptchaWidget

class ReCaptchaWidget(text.TextWidget):
    maxlength = 7
    size = 8

    zope.interface.implementsOnly(IReCaptchaWidget)

    def captchaImage(self):
        self.captcha = zope.component.getMultiAdapter((aq_inner(self.context), self.request), name='captcha')
        return self.captcha.image_tag()

    def captchaAudio(self):
        self.captcha = zope.component.getMultiAdapter((aq_inner(self.context), self.request), name='captcha')
        return self.captcha.audio_url()

def ReCaptchaFieldWidget(field, request):
    """IFieldWidget factory for CaptchaWidget."""
    return widget.FieldWidget(field, ReCaptchaWidget(request))